#ifndef TrackAnalysis_h
#define TrackAnalysis_h 1


#include <iostream>
#include <string>
#include <vector>


#include "marlin/Processor.h"
//#include <marlin/Exceptions.h>
//#include <marlin/Global.h>

#include "lcio.h"

#include <EVENT/LCCollection.h>
#include <EVENT/MCParticle.h>
#include <EVENT/SimTrackerHit.h>
#include <EVENT/SimCalorimeterHit.h>
#include <EVENT/TrackerHit.h>
#include <EVENT/CalorimeterHit.h>
#include <EVENT/Track.h>
#include <EVENT/ReconstructedParticle.h>
#include <UTIL/LCRelationNavigator.h>

#include <marlin/Global.h>
#include <gear/GEAR.h>
#include <gear/VXDParameters.h>
#include <gear/GearParameters.h>
#include <gear/VXDLayerLayout.h>
#include <gear/BField.h>


#include <MarlinUtil.h>

#include "/home/ilc/ilcsoft/v01-04/root/5.16.00/include/TH1F.h"
#include "/home/ilc/ilcsoft/v01-04/root/5.16.00/include/TFile.h"
#include "/home/ilc/ilcsoft/v01-04/root/5.16.00/include/TNtuple.h"
#include "/home/ilc/ilcsoft/v01-04/root/5.16.00/include/TVector3.h"

#ifdef MARLIN_USE_AIDA
#include <marlin/AIDAProcessor.h>
#include <AIDA/IHistogramFactory.h>
#include <AIDA/ICloud1D.h>
#include <AIDA/ICloud2D.h>
#endif


using namespace lcio ;
using namespace marlin ;


/**
 *    This processor provides an analysis of the tracking performance. The plots are arranged in the following different categories:
 *    <br>
 *    1. MC particle related plots <br>
 *    3. Plots related to the hits in tracking devices <br>
 *    4. Plots related to tracks <br>
 *    <br>
 *    Steering parameters: <br>
 *    FillMCGen: toggles the check plots for the MC particles with generator status == 1 ( 0 or 1 ) <br>
 *    FillMCSim : toggles the check plots for the MC particles with generator status != 1 ( 0 or 1 ) <br>
 *    ThetaCut : cut in theta to assign particles which are lost in the beam pipe
 *    FillTracks : toggles the check plots for the tracks
 *    ColNameTracks : name of the LCCollection of tracks
 *    ColNameRelationTrackToMCP : name of the LCRelation collection connecting the tracks and the corresponding MC particle
 *
 *    @author Marcel Vos (DESY)
 *    @version $Id: TrackAnalysis.h,v 1.3 2007/07/17 mvos Exp $
 *
 */
class TrackAnalysis : public Processor {
  
 public:
  
  virtual Processor*  newProcessor() { return new TrackAnalysis ; }
    
  TrackAnalysis() ;
  
  virtual void init() ;
  virtual void processRunHeader( LCRunHeader* run ) ;
  virtual void processEvent( LCEvent * evt ) ; 
  virtual void check( LCEvent * evt ) ; 
  virtual void end() ;
  
  void fillMCGenCheckPlots(LCEvent * evt);
  void fillMCSimCheckPlots(LCEvent * evt);
  void fillTrackFakeRate(LCEvent * evt);
  void fillTrackEfficiency(LCEvent * evt);

  void fillSimTrackerHitCheckPlots(LCEvent * evt);


  void fillTrackerHitCheckPlots(LCEvent * evt);
  


  
  void fillComparisonMCRecoPlots();

 private:

  int _nRun;
  int _nEvt;

  float _bField;

  int _fillMCGen;
  float _thetaCut;
  int _fillMCSim;


  int _fillTracks;
  std::string _colNameTracks;
  std::string _colNameRelationTrackToMCP; 
  std::string _colNameRelationMCToTrack;

  int _fillReconstructedParticles;
  std::string _colNameReconstructedParticles;

  int _fillComparisonMCReco;
  int _nMC;
  int _nMCCh;
  int _nMCN;
  int _nReco;
  int _nRecoCh;
  int _nRecoN;

  double _energyMC;
  double _energyMCCh;
  double _energyMCN;
  double _energyReco;
  double _energyRecoCh;
  double _energyRecoN;


  TH1F* kk;
  TNtuple* track_eff_ntuple;
  TNtuple* track_fake_ntuple;
  TNtuple* interactions_ntuple;
  TNtuple* interaction_prob_ntuple;


  
} ;

#endif



