#include "TrackAnalysis.h"





using namespace lcio ;
using namespace marlin ;




TrackAnalysis aTrackAnalysis ;



TrackAnalysis::TrackAnalysis() : Processor("TrackAnalysis") {

  std::cout << " TrackAnalysis::TrackAnalysis " << std::endl;

  _nMC = 0;
  _nMCCh = 0;
  _nMCN = 0;
  _nReco = 0;
  _nRecoCh = 0;
  _nRecoN = 0;
  
  _energyMC = 0.0;
  _energyMCCh = 0.0;
  _energyMCN = 0.0;
  _energyReco = 0.0;
  _energyRecoCh = 0.0;
  _energyRecoN = 0.0;
  

  _description = "Analyzes track performance" ;


    
  registerProcessorParameter("FillMCGen","fill clouds for MC particles generated at IP",
			     _fillMCGen,
			     (int)0);
  
  registerProcessorParameter("FillMCSim","fill clouds for MC particles created during simulation",
			     _fillMCSim,
			     (int)0);

  

  registerProcessorParameter("ThetaCut","Polar angle cut, given in rad. Particles with a smaller polar angle will be treated as 'lost in the beam pipe'",
			     _thetaCut,
			     (float)0.1);

    
  registerProcessorParameter("FillTracks","fill clouds for tracks",
			     _fillTracks,
			     (int)1);

  registerProcessorParameter("ColNameTracks" ,
			     "name of the Track collection" ,
			     _colNameTracks ,
			     std::string("Tracks") ); 

  registerProcessorParameter( "ColNameRelationTrackToMCP" , 
			      "name of the LC Relation collection between Tracks and MC particles"  ,
			      _colNameRelationTrackToMCP,
			      std::string("TrueTrackToMCP") );

   registerProcessorParameter( "ColNameRelationMCToTrack" , 
			      "name of the LC Relation collection between MC particles and tracks"  ,
			      _colNameRelationMCToTrack,
			      std::string("TrueMCToTrack") );



  registerProcessorParameter("Fill","fill clouds for comparison of MC tree and reconstructed particles",
			     _fillComparisonMCReco,
			     (int)1);

}




void TrackAnalysis::init() { 

  printParameters();

  std::cout << " TrackAnalysis::init " << std::endl;
  const gear::TPCParameters& gearTPC = Global::GEAR->getTPCParameters() ;
  

  _bField = Global::GEAR->getBField().at( gear::Vector3D( 0., 0., 0.) ).z() ;

  _nRun = -1;
  _nEvt = 0;

  std::cout << " booking " << std::endl;
  kk = new TH1F("kk","kk",100,0,100);
  track_eff_ntuple = new TNtuple("track_eff","track_eff","pdg:pt:p:theta:charge:pDec:dau:genstat:simstat:time:vtxx:vtxy:vtxz:evtxx:evtxy:evtxz:wei");
  track_fake_ntuple = new TNtuple("track_fake","track_fake","pt:p:theta:phi:nhit:nvtx:nsit:nftd:ntpc:wei");
  interactions_ntuple = new TNtuple("interactions","interactions","pdg:pt:p:theta:ppdg:ppt:pp:ptheta:time:vtxx:vtxy:vtxz:evtxx:evtxy:evtxz");
  interaction_prob_ntuple = new TNtuple("interaction_prob","interaction_prob","pdg:pt:p:theta:dpdg:dpt:dp:dtheta:time:vtxx:vtxy:vtxz:evtxx:evtxy:evtxz");

}



void TrackAnalysis::processRunHeader( LCRunHeader* run) { 

  _nRun++ ;
  _nEvt = 0;

} 




void TrackAnalysis::processEvent( LCEvent * evt ) { 

  
  std::cout << " TrackAnalysis::event " << std::endl;
  static bool firstEvent = true;
  
  if(firstEvent) std::cout << "Check Plot processor called for first event" << std::endl;


  if (_fillMCGen) fillMCGenCheckPlots(evt);
  if (_fillMCSim) fillMCSimCheckPlots(evt);

  fillSimTrackerHitCheckPlots(evt);
    
  fillTrackerHitCheckPlots(evt);
    
  if (_fillTracks) fillTrackFakeRate(evt);

  if (_fillComparisonMCReco) fillTrackEfficiency(evt);
  

  _nEvt ++;
  firstEvent = false;
 
}




void TrackAnalysis::check( LCEvent * evt ) { 
  
}




void TrackAnalysis::end(){ 

  TFile *hfile = new TFile("out.root","RECREATE");
  kk->Write();
  track_eff_ntuple->Write();
  track_fake_ntuple->Write();
  interactions_ntuple->Write(); 
  interaction_prob_ntuple->Write();
  hfile->Close();
  std::cout << " end, storing results " << std::endl;

}






void TrackAnalysis::fillTrackerHitCheckPlots(LCEvent * evt)
{
  std::cout << "Hoer " << std::endl;
}

void TrackAnalysis::fillSimTrackerHitCheckPlots(EVENT::LCEvent*)
{
  std::cout << "Hoera " << std::endl;
}




void TrackAnalysis::fillTrackEfficiency(LCEvent * evt){

  try {
    LCCollection* LCRcolTracks = evt->getCollection(_colNameRelationTrackToMCP);
    LCRelationNavigator* navTracks = new LCRelationNavigator(LCRcolTracks);
    
    
    try {
    
      const std::vector< std::string >* strVec = evt->getCollectionNames() ;
      std::vector< std::string >::const_iterator name ;
      
      for( name = strVec->begin() ; name != strVec->end() ; name++) {
	
	LCCollection* col = evt->getCollection( *name ) ;
	
	if ( col->getTypeName() == LCIO::MCPARTICLE ) {
	  
	  int nMCP = col->getNumberOfElements();
	  std::cout << " looping over " << nMCP << " Monte Carlo particles " << std::endl;
	
	  for(int i = 0; i < nMCP ; ++i){
	    
	    MCParticle* mcp = dynamic_cast<MCParticle*>( col->getElementAt( i ) ) ;
	    int pdg = mcp->getPDG();
	    bool parentDecayed = mcp->vertexIsNotEndpointOfParent();
	    int charge = mcp->getCharge();
	    const double* p = mcp->getMomentum();
	    
	    double mag = sqrt(p[0]*p[0] + p[1]*p[1] + p[2]*p[2]);
	    double pt = hypot(p[0],p[1]);
	    double theta = atan2(pt,p[2]);
	    
	    const double * vtx = mcp->getVertex();
	    const double * endvtx = mcp->getEndpoint();
	    
	    vector<MCParticle*> daughters = mcp->getDaughters();
	    int status1 = mcp->getGeneratorStatus();
	    int status2 = mcp->getSimulatorStatus();

	    //	    if (pt > 0.1 &&  fabs(theta) > _thetaCut && fabs(theta) < 3.14 - _thetaCut && charge != 0)
	    if (charge != 0) 
	      {
		const LCObjectVec& relTrackToMC = navTracks->getRelatedFromObjects(mcp); 
		vector<float> weights = navTracks->getRelatedFromWeights(mcp);
		
		int nOfRelatedTracks = relTrackToMC.size();
		std::cout << " this particle " << mcp->getPDG() << " has " << nOfRelatedTracks << " related tracks " << std::endl;
		float w=0;
		float time = mcp->getTime();
		for (int j=0;j<nOfRelatedTracks;j++)
		  {
		    if (weights[j]>w) w=weights[j];
		    
		  }
		float val[17];
		val[0]=pdg;
		val[1]=pt;
		val[2]=mag;
		val[3]=theta;
		val[4]=charge;
		val[5]=parentDecayed;
		val[6]=daughters.size();
		val[7]=status1;
		val[8]=status2;
		val[9]=time;
		val[10]=vtx[0];
		val[11]=vtx[1];
		val[12]=vtx[2];
		val[13]=endvtx[0];
		val[14]=endvtx[1];
		val[15]=endvtx[2];
		val[16]=w;
		track_eff_ntuple->Fill(val);
	      }
	  }
	}
      }
    }
    
    
    catch(DataNotAvailableException &e){
      std::cout << "MC particle collection not available in Check Plot processor" << std::endl ;
    }
  }
  catch(DataNotAvailableException &e){
    std::cout << "MC to track relations not available: TrackAnalysis processor" << std::endl ;
  }
}

void TrackAnalysis::fillMCGenCheckPlots(LCEvent * evt){


  if (_fillMCGen) {
    
    unsigned int NMCGen             = 0;
	  
    unsigned int NMCHChGen          = 0;
    unsigned int NMCH0Gen           = 0;
    unsigned int NMCGGen            = 0;
	  
    unsigned int NMCElectronsGen    = 0;
    unsigned int NMCMuonsGen        = 0;
    unsigned int NMCTausGen         = 0;
    unsigned int NMCNusGen          = 0;
    
    unsigned int NMCPiChGen         = 0;
    unsigned int NMCKChGen          = 0;
    unsigned int NMCProtonsGen      = 0;
    unsigned int NMCPi0Gen          = 0;
    unsigned int NMCK0lGen          = 0;
    unsigned int NMCK0sGen          = 0;
    unsigned int NMCNeutronsGen     = 0;
    unsigned int NMCGammasGen       = 0;
    unsigned int NMCLambda0sGen     = 0;
    unsigned int NMCSigma0sGen      = 0;
    unsigned int NMCXi0sGen         = 0;

    unsigned int NMCLostInBeamPipe  = 0;	  
    unsigned int NMCRemainingGen    = 0;
    
    double energyGen                = 0.0;
    double energyMCHChGen           = 0.0;
    double energyMCH0Gen            = 0.0;
    double energyMCGGen             = 0.0;
    double energyLostInBeamPipe     = 0.0;

       
    try {
      
      const std::vector< std::string >* strVec = evt->getCollectionNames() ;
      std::vector< std::string >::const_iterator name ;
    
      for( name = strVec->begin() ; name != strVec->end() ; name++) {
      
	LCCollection* col = evt->getCollection( *name ) ;

	if ( col->getTypeName() == LCIO::MCPARTICLE ) {

	  int nMCP = col->getNumberOfElements();
	  
	  for(int i = 0; i < nMCP ; ++i){
	    
	    MCParticle* mcp = dynamic_cast<MCParticle*>( col->getElementAt( i ) ) ;
	   	    
	    const double* p = mcp->getMomentum();
	    float e = mcp->getEnergy();

	    if ( mcp->getGeneratorStatus() == 1 ) {

	      double pt = hypot(p[0],p[1]);
	      double theta = atan2(pt,p[2]);

	      if ( fabs(theta) > _thetaCut && fabs(theta) < 3.14 - _thetaCut ) {
		
		++NMCGen;
		energyGen += e;

		
		switch (abs(mcp->getPDG())) {
		  
		case 11  : {
		  ++NMCElectronsGen;



		  ++NMCHChGen;
		  energyMCHChGen += e;
		  break;
		}
		case 13  : {
		  ++NMCMuonsGen;


		  ++NMCHChGen;
		  energyMCHChGen += e;
		  break;
		}
		case 15  : {
		  ++NMCTausGen;


		  ++NMCHChGen;
		  energyMCHChGen += e;
		  break;
		}
		case  12 : {
		  ++NMCNusGen;


		  break;
		}
		case  14 : {
		  ++NMCNusGen;


		  break;
		}
		case  16 : {
		  ++NMCNusGen;


		  break;
		}
		  
		case 211  : {
		  ++NMCPiChGen;


		  ++NMCHChGen;
		  energyMCHChGen += e;
		  break;
		}
		case 321  : {
		  ++NMCKChGen;


		  ++NMCHChGen;
		  energyMCHChGen += e;
		  break;
		}
		case 2212  : {
		  ++NMCProtonsGen;

		  ++NMCHChGen;
		  energyMCHChGen += e;
		  break;
		}
		case 111  : {
		  ++NMCPi0Gen;
		  ++NMCGGen;
		  energyMCGGen += e;

		  break;
		}
		case 130 : {
		  ++NMCK0lGen;


		  ++NMCH0Gen;
		  energyMCH0Gen += e;
		  break;
		}
		case 310  : {
		  ++NMCK0sGen;


		  ++NMCHChGen;
		  energyMCHChGen += e;
		  break;
		}
		case 2112 : {
		  ++NMCNeutronsGen;



		  ++NMCH0Gen;
		  energyMCH0Gen += e;
		  break;
		}
		case 22 : {
		  ++NMCGammasGen;
		  ++NMCGGen;
		  energyMCGGen += e;

		  break;
		}
		case 3122 : {
		  ++NMCLambda0sGen;


		  ++NMCHChGen;
		  energyMCHChGen += e;
		  break;
		}
		case 3212 : {
		  ++NMCSigma0sGen;


		  ++NMCHChGen;
		  energyMCHChGen += e;
		  break;
		}
		case 3322 : {
		  ++NMCXi0sGen;


		  ++NMCHChGen;
		  energyMCHChGen += e;
		  break;
		}

		default  : {

		  // debug
		  std::cout << "default case for for MCP (generator status == 1)" << mcp->getPDG() << std::endl;
		  
		  ++NMCRemainingGen;


		  ++NMCHChGen;
		  energyMCHChGen += e;
		  break;
		}
		  
		} // end of switch
		
	      }
	      else { // in beam pipe

		++NMCGen;
		energyGen += e;

		++NMCLostInBeamPipe;
		energyLostInBeamPipe += e;
		
	      }
	      
	    }
	    
	  }
	  		  
	}
	
      }
          
    }
    catch(DataNotAvailableException &e){
      std::cout << "MC particle collection not available in Check Plot processor" << std::endl ;
    };


    _nMC = NMCGen - NMCLostInBeamPipe;
    _nMCCh = NMCElectronsGen + NMCMuonsGen + NMCTausGen + NMCHChGen;
    _nMCN = NMCH0Gen + NMCGGen;
    
    _energyMC = energyGen - energyLostInBeamPipe;
    _energyMCCh = energyMCHChGen;
    _energyMCN  = energyMCH0Gen + energyMCGGen;

    std::cout << " generator " << _nMC << " " << _nMCCh << " " << _nMCN << std::endl;
    std::cout << " generator " << _energyMC << " " << _energyMCCh << " " << _energyMCN << std::endl;

  }

}









void TrackAnalysis::fillMCSimCheckPlots(LCEvent * evt){


  if (_fillMCSim) {
	  
    unsigned int NMCSim             = 0; 
    unsigned int NMCSimRelevant     = 0;
    
    unsigned int NMCElectronsSim    = 0;
    unsigned int NMCMuonsSim        = 0;
    unsigned int NMCTausSim         = 0;
    unsigned int NMCNusSim          = 0;
    
    unsigned int NMCPiChSim         = 0;
    unsigned int NMCKChSim          = 0;
    unsigned int NMCProtonsSim      = 0;
    unsigned int NMCPi0Sim          = 0;
    unsigned int NMCK0lSim          = 0;
    unsigned int NMCK0sSim          = 0;
    unsigned int NMCNeutronsSim     = 0;
    unsigned int NMCGammasSim       = 0;
    unsigned int NMCLambda0sSim     = 0;
    unsigned int NMCSigma0sSim      = 0;
    unsigned int NMCXi0sSim         = 0;
    
    unsigned int NMCRemainingSim    = 0;
    
    double energySim                = 0.0;
	  

    try {
      
      const std::vector< std::string >* strVec = evt->getCollectionNames() ;
      std::vector< std::string >::const_iterator name ;
      
      for( name = strVec->begin() ; name != strVec->end() ; name++) {
	
	LCCollection* col = evt->getCollection( *name ) ;
	
	if ( col->getTypeName() == LCIO::MCPARTICLE ) {
	  
	  int nMCP = col->getNumberOfElements();
	  
	  for(int i = 0; i < nMCP ; ++i){
	    
	    MCParticle* mcp = dynamic_cast<MCParticle*>( col->getElementAt( i ) ) ;
	    
	    float e = mcp->getEnergy();
	    
	    if (mcp->getGeneratorStatus() != 1 ) {
	      
	      ++NMCSim;
	      energySim += e;
		
	      int pdg = mcp->getPDG();
	      bool parentDecayed = mcp->vertexIsNotEndpointOfParent();
	      int charge = mcp->getCharge();
	      const double* p = mcp->getMomentum();

	      TVector3 vect(p[0],p[1],p[2]);
	      

	      double mag = sqrt(p[0]*p[0] + p[1]*p[1] + p[2]*p[2]);
	      double pt = hypot(p[0],p[1]);
	      double theta = atan2(pt,p[2]);
	      double theta2 = vect.Theta();
	      
	      const double * vtx = mcp->getVertex();
	      const double * endvtx = mcp->getEndpoint();
	      
	      vector<MCParticle*> daughters = mcp->getDaughters();
	      int status1 = mcp->getGeneratorStatus();
	      int status2 = mcp->getSimulatorStatus();

	      
	      float time = mcp->getTime();
	      if (sqrt(vtx[0]*vtx[0]+vtx[1]*vtx[1]+vtx[2]*vtx[2]) < 1400)
		{
		  vector<MCParticle*> parents = mcp->getParents();
		  for (int j=0;j<parents.size();j++)
		    {
		      
		      if (abs(mcp->getPDG()) == 11 && parents[j]->getPDG() == 22) std::cout << " interactions: conversion " << parents[j]->getEnergy() << std::endl;
		      if (abs(mcp->getPDG()) == 211 && abs(parents[j]->getPDG()) == 211) std::cout << " interactions: nuclear interaction " << mcp->getEnergy() << " " << parents[j]->getEnergy() << std::endl;
		      if (abs(mcp->getPDG()) == 211 && abs(parents[j]->getPDG()) != 211) std::cout << " interactions: hunk " << parents.size() << std::endl;
		      if (abs(mcp->getPDG()) == 22 && abs(parents[j]->getPDG()) == 11) std::cout << " interactions: brehmsstrahlung " << mcp->getEnergy() << std::endl;	 
		      const double* p2 = parents[j]->getMomentum(); 
		      double mag2 = sqrt(p2[0]*p2[0] + p2[1]*p2[1] + p2[2]*p2[2]);
		      double pt2 = hypot(p2[0],p2[1]);
		      double theta2 = atan2(pt2,p2[2]);
		      float val[17];
		      val[0]=pdg;
		      val[1]=pt;
		      val[2]=mag;
		      val[3]=theta;
		      val[4]=parents[j]->getPDG();
		      val[5]=pt2;
		      val[6]=mag2;
		      val[7]=theta2;
		      val[8]=time;
		      val[9]=vtx[0];
		      val[10]=vtx[1];
		      val[11]=vtx[2];
		      val[12]=endvtx[0];
		      val[13]=endvtx[1];
		      val[14]=endvtx[2];
		      
		      
		      interactions_ntuple->Fill(val);
		    }
		}

	      if (mag > 0.5 &&  fabs(theta) > _thetaCut && fabs(theta) < 3.14 - _thetaCut )
		{
		  std::cout << " found relevant GEANT particle " << mag << " " << theta << std::endl;
		  ++NMCSimRelevant;
		}

		
	      switch (abs(mcp->getPDG())) {
		  
	      case 11  : {
		++NMCElectronsSim;

		break;
	      }
	      case 13  : {
		++NMCMuonsSim;

		break;
	      }		
	      case 15  : {
		++NMCTausSim;

		break;
	      }
	
	      case  12 : {
		++NMCNusSim;


		break;
	      }
	      case  14 : {
		++NMCNusSim;


		break;
	      }
	      case 16 : {
		++NMCNusSim;


		break;
	      }
		  
	      case 211  : {
		++NMCPiChSim;

		break;
	      }
	      case 321  : {
		++NMCKChSim;


		break;
	      }
	      case 2212  : {
		++NMCProtonsSim;


		break;
	      }
	      case 111  : {
		++NMCPi0Sim;

		break;
	      }
	      case 130 : {
		++NMCK0lSim;

		break;
	      }
	      case 310  : {
		++NMCK0sSim;

		break;
	      }
	      case 2112 : {
		++NMCNeutronsSim;

		break;
	      }
	      case 22  : {
		++NMCGammasSim;


		break;
	      }
	      case 3122 : {
		++NMCLambda0sSim;

		break;
	      }
	      case 3212 : {
		++NMCSigma0sSim;
		  
		break;
	      }
	      case 3322 : {
		++NMCXi0sSim;

		break;
	      }
	
	      default  : {

		// debug
		std::cout << "default case for for MCP (generator status != 1)" << mcp->getPDG() << std::endl;
		  
		++NMCRemainingSim;

	 	break;
	      }
		
	      } // end of switch
	      
	    }
	    else
	      {  
		int pdg = mcp->getPDG();
		if (abs(pdg) == 11 || abs(pdg)==211 || abs(pdg) == 22)
		  {
		    bool parentDecayed = mcp->vertexIsNotEndpointOfParent();
		    int charge = mcp->getCharge();
		    const double* p = mcp->getMomentum();
		    
		    double mag = sqrt(p[0]*p[0] + p[1]*p[1] + p[2]*p[2]);
		    double pt = hypot(p[0],p[1]);
		    double theta = atan2(pt,p[2]);
		    
		    const double * vtx = mcp->getVertex();
		    const double * endvtx = mcp->getEndpoint();
		    float time = mcp->getTime();
		    vector<MCParticle*> daughters = mcp->getDaughters();
		    double mag2=0;
		    double pt2=0;
		    double theta2=0;
		    int pdg2=0;
		    if (daughters.size() != 0 && sqrt(endvtx[0]*endvtx[0]+endvtx[1]*endvtx[1]+endvtx[2]*endvtx[2]) < 1400)
		      {
			MCParticle *daughter = daughters[0]; 
			const double* p2 = daughter->getMomentum();
		    
			mag2 = sqrt(p2[0]*p2[0] + p2[1]*p2[1] + p2[2]*p2[2]);
			pt = hypot(p2[0],p2[1]);
			theta2 = atan2(pt2,p2[2]);
			int pdg2 = daughter->getPDG();
		      }
		    float val[17];
		    val[0]=pdg;
		    val[1]=pt;
		    val[2]=mag;
		    val[3]=theta;
		    val[4]=pdg2;
		    val[5]=pt2;
		    val[6]=mag2;
		    val[7]=theta2;
		    val[8]=time;
		    val[9]=vtx[0];
		    val[10]=vtx[1];
		    val[11]=vtx[2];
		    val[12]=endvtx[0];
		    val[13]=endvtx[1];
		    val[14]=endvtx[2];
		    
		    
		    
		    interaction_prob_ntuple->Fill(val);
		      
		  }
	      }
	    
	  }
	  
	}

      }

    }
    catch(DataNotAvailableException &e){
      std::cout << "MC particle collection not available in Check Plot processor" << std::endl ;
    };
    


    std::cout << " Found " << NMCSim << " GEANT4 particles " << std::endl;
    std::cout << " of which " << NMCSimRelevant << " are relevant " << std::endl;
    
  }

}





void TrackAnalysis::fillTrackFakeRate(LCEvent * evt) {


  if (_fillTracks) {

    try {
    
      std::vector< std::string >::const_iterator iter;
      const std::vector< std::string >* ColNames = evt->getCollectionNames();
    
      for( iter = ColNames->begin() ; iter != ColNames->end() ; iter++) {
      
	LCCollection* col = evt->getCollection( *iter ) ;
  
	if ( (col->getTypeName() == LCIO::TRACK) && (*iter == _colNameTracks) ) {

	  int nTracks = col->getNumberOfElements();
	  std::cout << " found " << nTracks << " reconstructed tracks " << std::endl;

	  for(int j=0; j<nTracks; ++j){
	  
	    Track* track = dynamic_cast<Track*>(col->getElementAt(j));

	    int nTrackerHits = track->getTrackerHits().size();
	    nTrackerHits = track->getSubdetectorHitNumbers().size();
	    std::cout << " hoera " << nTrackerHits << " " << track->getSubdetectorHitNumbers()[0] << " " << track->getSubdetectorHitNumbers()[1] << " " << track->getSubdetectorHitNumbers()[2] << " " << track->getSubdetectorHitNumbers()[3] << std::endl;
	    int vtxhits = track->getSubdetectorHitNumbers()[0];
	    int sithits = track->getSubdetectorHitNumbers()[1];
	    int ftdhits = track->getSubdetectorHitNumbers()[2];
	    int tpchits =  track->getSubdetectorHitNumbers()[3];
	    const double absP = MarlinUtil::getAbsMomentum(track,_bField);
	    const double* p = MarlinUtil::getMomentum(track,_bField);
	    const double pt = sqrt( p[0]*p[0] + p[1]*p[1] );
	    double theta = atan2(pt,p[2]);
	    TVector3 vect(p[0],p[1],p[2]);
	      

	     
	    double theta2 = vect.Theta();

	    std::cout << " track " << pt << " " << absP << " " << theta << " " << " " << theta2 << " " << _bField << std::endl; 
	    kk->Fill(absP);
	    //std::cout << " filling " << std::endl;
	    float  w=0;
	    try {

	      LCCollection* LCRcolTracks = evt->getCollection(_colNameRelationTrackToMCP);

	      LCRelationNavigator* navTracks = new LCRelationNavigator(LCRcolTracks);
	      const LCObjectVec& relMCParticlesToTrack = navTracks->getRelatedToObjects(track); 
	      vector<float> weights = navTracks->getRelatedFromWeights(track);
	      int nOfRelatedMCParticles = relMCParticlesToTrack.size();
	      
	      std::cout << " this track has " << nOfRelatedMCParticles << " related MC Particles " << std::endl;
	      for (int i=0;i<nOfRelatedMCParticles;i++)
		{
		  std::cout << " particle " << i << " related with weight " << weights[i] << std::endl;
		  if (weights[i]>w) w=weights[i];
		}
	     
	    }
	    catch(DataNotAvailableException &e){std::cout << "no valid LCRelation between track and MC particle in event " << _nEvt << std::endl; };
	    //pt:p:theta:phi:nhit:wei
	    std::cout << " FILL FAKE ntup " << pt << " " << absP << " " << theta << std::endl;
	    track_fake_ntuple->Fill(pt,absP,theta,0,nTrackerHits,vtxhits,sithits,ftdhits,tpchits,w);
 
	  }
	  
	}
	
      }
      
    }
    catch(DataNotAvailableException &e){std::cout << "no valid track collection available in event " << _nEvt << std::endl; };
  }

}









void TrackAnalysis::fillComparisonMCRecoPlots() {

  if (_fillComparisonMCReco) {

    if ( _fillMCGen && _fillReconstructedParticles ) {


    }

  }

}
